/*
 * File: YarnPattern.js
 * --------------------
 * This program uses the GLine class to simulate the process of winding a
 * piece of colored yarn around a set of pegs equally spaced along the
 * perimeter of a rectangle.  At each step, the yarn is stretched from
 * its current peg to the one DELTA pegs further on.
 */

/* Constants */

const N_ACROSS = 80;
const N_DOWN = 50;
const DELTA = 97;

/*
 * Creates a pattern that simulates the process of winding a piece of
 * yarn around an array of pegs along the edges of the canvas with the
 * id "canvas".
 */

function drawYarnPattern() {
   var canvas = document.getElementById("canvas");
   var ctx = canvas.getContext("2d");
   var pegs = createPegArray(canvas.width, canvas.height, N_ACROSS, N_DOWN);
   ctx.moveTo(pegs[0].x, pegs[0].y);
   var thisPeg = 0;
   var nextPeg = -1;
   while (thisPeg !== 0 || nextPeg === -1) {
      nextPeg = (thisPeg + DELTA) % pegs.length;
      var pt = pegs[nextPeg];
      ctx.lineTo(pt.x, pt.y);
      thisPeg = nextPeg;
   }
   ctx.stroke();
}

/*
 * Creates a new Point object.  If this function is called with no
 * arguments, it creates a Point object at the origin.
 */

function Point(x, y) {
   if (x === undefined) {
      x = 0;
      y = 0;
   }
   return { x: x, y: y };
}

/*
 * Creates an array of pegs around the perimeter of a rectangle with the
 * specified width and height.  The number of pegs in each dimension is
 * given by the values nAcross and nDown.
 */

function createPegArray(width, height, nAcross, nDown) {
   var dx = width / nAcross;
   var dy = height / nDown;
   var pegs = [ ];
   for (var i = 0; i < nAcross; i++) {
      pegs.push(Point(i * dx, 0));
   }
   for (var i = 0; i < nDown; i++) {
      pegs.push(Point(nAcross * dx, i * dy));
   }
   for (var i = nAcross; i > 0; i--) {
      pegs.push(Point(i * dx, nDown * dy));
   }
   for (var i = nDown; i > 0; i--) {
      pegs.push(Point(0, i * dy));
   }
   return pegs;
}
